#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
reload(sys)
sys.setdefaultencoding('utf8')

import os
import dbus
import dbus.service
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from service.dbus_service import DbusService
from service.kylin_update_service import KylinUpdateService
from utils.enums import LaunchMode, TaskMode, LWMode, AUPSource, AppType, AppMsg, Signals, LOADINGWIDTH, LOADINGHEIGHT, MAINWIDTH, MAINHEIGHT, DETAILWIDTH, DETAILHEIGHT, SINTERFACE, SINTERFACES
from utils.common_utils import CommonUtils
from utils.util_dbus import UtilDbus
from utils.log_manager import LogManager
from utils import log_manager
from mainui.updatelistitemwidget import UpdateListItemWidget
from mainui.messagewidget import MessageWidget
from mainui.loadingwidget import LoadingWidget
from mainui.topwidget import TopWidget
from mainui.bottomwidget import BottomWidget
from mainui.detailwidget import DetailWidget
from mainui.system_tray import SystemTray


class MainWindow(QWidget):

    service_key = ""
    hash_pwd = ""
    item_map_ordinary = {}
    item_map_important = {}

    size_ordinary = 0
    size_important = 0
    app_num_ordinary = 0
    app_num_important = 0
    selected_num_ordinary = 0
    selected_num_important = 0

    available_source = AUPSource.NOSOURCE
    is_working = False
    is_working_c = False
    launch_mode = None
    task_mode = None

    iface = None
    kus = None
    cu = None

    previous_process_type = ""

    def __init__(self, launch_mode, parent=None):
        QWidget.__init__(self, parent)

        self.launch_mode = launch_mode

        self.init_log()
        self.init_ui()
        self.check_singleton()
        self.cu = CommonUtils()
        self.check_service_key()
        self.iface = DbusService()
        self.kus = KylinUpdateService(self)
        self.init_main_animation()
        self.init_connect()

        self.pre_work()

    # 初始化UI
    def init_ui(self):
        self.setWindowIcon(QIcon("./res/kylin-update-manager.png"))
        self.setWindowTitle("麒麟更新管理器")
        self.setWindowOpacity(1)
        self.setAutoFillBackground(True)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Widget)

        self.top_widget = TopWidget(self)
        self.bottom_widget = BottomWidget(self)
        self.detail_widget = DetailWidget(self)
        self.detail_widget.move(-336, 0)
        self.loading_widget = LoadingWidget(self)

        self.mouse_press = False
        self.animation_msg_box = MessageWidget("发现需要更新的软件。", False)

        # 预先调用QApp的show，否则在一次信号传递后QApp会自动退出
        self.resize(0, 0)
        self.show()

        if(self.launch_mode == LaunchMode.MANUAL):
            self.resize(LOADINGWIDTH, LOADINGHEIGHT)
            self.loading_widget.progressBar.hide()
            self.loading_widget.loadinglabel.show()
            self.move_to_center()
            self.show_to_frontend()

    # 初始化动画效果
    def init_main_animation(self):
        topAnimation = QPropertyAnimation(self.top_widget, "pos")
        topAnimation.setDuration(1000)
        topAnimation.setStartValue(QPoint(0, -141))
        topAnimation.setEndValue(QPoint(0, 0))

        bottomAnimation = QPropertyAnimation(self.bottom_widget, "pos")
        bottomAnimation.setDuration(1000)
        bottomAnimation.setStartValue(QPoint(0, 600))
        bottomAnimation.setEndValue(QPoint(0, 141))

        loadingAnimation = QPropertyAnimation(self.loading_widget, "pos")
        loadingAnimation.setDuration(1000)
        loadingAnimation.setStartValue(QPoint(0, 0))
        loadingAnimation.setEndValue(QPoint(672, 0))

        self.mainGroup = QParallelAnimationGroup()
        self.mainGroup.addAnimation(topAnimation)
        self.mainGroup.addAnimation(bottomAnimation)
        self.mainGroup.addAnimation(loadingAnimation)

        topOrigAnimation = QPropertyAnimation(self.top_widget, "pos")
        topOrigAnimation.setDuration(1000)
        topOrigAnimation.setStartValue(QPoint(0, 0))
        topOrigAnimation.setEndValue(QPoint(0, -141))

        bottomOrigAnimation = QPropertyAnimation(self.bottom_widget, "pos")
        bottomOrigAnimation.setDuration(1000)
        bottomOrigAnimation.setStartValue(QPoint(0, 141))
        bottomOrigAnimation.setEndValue(QPoint(0, 600))

        detailOrigAnimation = QPropertyAnimation(self.detail_widget, "pos")
        detailOrigAnimation.setDuration(1000)
        detailOrigAnimation.setStartValue(QPoint(-336, 0))
        detailOrigAnimation.setEndValue(QPoint(0, 0))

        self.detailGroup = QParallelAnimationGroup(self)
        self.detailGroup.addAnimation(topOrigAnimation)
        self.detailGroup.addAnimation(bottomOrigAnimation)
        self.detailGroup.addAnimation(detailOrigAnimation)

    # 信号槽绑定
    def init_connect(self):
        self.loading_widget.title_bar.min_button.clicked.connect(self.slot_click_min_button)
        self.loading_widget.title_bar.close_button.clicked.connect(self.slot_click_close_button)
        self.top_widget.btn_min.clicked.connect(self.slot_click_min_button)
        self.top_widget.btn_close.clicked.connect(self.slot_click_close_button)
        self.top_widget.btn_install.clicked.connect(self.slot_click_install_button)
        self.bottom_widget.lw_ordinary.itemClicked.connect(self.slot_click_update_item)
        self.bottom_widget.lw_important.itemClicked.connect(self.slot_click_update_item)
        self.bottom_widget.click_ordinary_main_cb.connect(self.slot_reset_ordinary_cb)
        self.bottom_widget.click_important_main_cb.connect(self.slot_reset_important_cb)
        self.detail_widget.title_bar.min_button.clicked.connect(self.slot_click_min_button)
        self.detail_widget.title_bar.close_button.clicked.connect(self.slot_click_close_button)

        self.animation_msg_box.quitSignal.connect(self.exit_app)
        self.connect(self.iface, Signals.dbus_process_msg, self.slot_dbus_process_msg)

    # 初始化日志模块
    def init_log(self):
        log_manager.init_log()

    # 保证单例模式
    def check_singleton(self):
        try:
            bus = dbus.SessionBus()
        except:
            LogManager.CRI("无法初始化dbus，程序退出。")
            sys.exit(0)

        # 已有实例在运行，将其显示到前端
        try:
            proxy_obj = bus.get_object(SINTERFACE, SINTERFACES)
            iface = dbus.Interface(proxy_obj, SINTERFACE)
            iface.bring_to_front()
            # 退出重复运行的当前实例
            sys.exit(0)

        # 没有实例在运行，创建一个dbus
        except dbus.DBusException:
            bus_name = dbus.service.BusName(SINTERFACE, bus)
            self.dbusControler = UtilDbus(self, bus_name)

    # 获取麒麟操作系统服务序列号
    def check_service_key(self):
        print "正在获取服务序列号..."
        self.service_key = self.cu.get_service_key()
        self.hash_pwd = self.cu.get_hash_pwd()

        if self.service_key is None:
           os.system("kylin-verify-gui")
           LogManager.ERR("无法获取操作系统服务序列号，程序退出！")
           self.exit_app("无法获取操作系统服务序列号，程序退出！")

    # 准备工作
    def pre_work(self):
        # 修复可能出现的dpkg问题，完成后由slot_dbus_process_msg() 执行 main_work()
        print "正在执行dpkg configure."
        self.iface.call_configure_dpkg_by_shell()

    # 主流程
    def main_work(self):
        if(self.kus.check_available_medium() == True):
            if(self.kus.is_sinfo_down_success == True):
                if(self.kus.is_sinfo_match == True):
                    self.check_status()
                else:
                    qb = QMessageBox(self)
                    qb.setIcon(QMessageBox.Question)
                    qb.setWindowTitle("没有匹配的源")
                    qb.setText("当前连接的服务器或光盘中没有完全匹配的软件源。\n是否选择其中的默认软件源进行升级？\n\n这可能会导致未知的后果，请向技服人员确认。")
                    qb.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
                    qb.setButtonText(QMessageBox.Yes, QString("确 认"))
                    qb.setButtonText(QMessageBox.No, QString("取 消"))
                    rtn = qb.exec_()

                    if(rtn == QMessageBox.Yes):
                        self.check_status()
                    else:
                        self.set_default_sourceslist()
                        self.exit_app("用户取消，程序退出.")

            # 下载sinfo失败
            else:
                self.set_default_sourceslist()

                if(self.kus.is_sinfo_match == False):
                    if(self.launch_mode == LaunchMode.AUTOMATIC):
                        self.exit_app("媒介中没有可用的软件源，程序退出.")
                    else:
                        self.animation_msg_box.set_animation_args("媒介中没有可用的软件源，程序退出", True)
                        self.animation_msg_box.showAnimation(5000)
                else:
                    if(self.launch_mode == LaunchMode.AUTOMATIC):
                        self.exit_app("下载软件源信息出错，程序退出.")
                    else:
                        self.animation_msg_box.set_animation_args("下载软件源信息出错，程序退出.", True)
                        self.animation_msg_box.showAnimation(5000)

        # 找不到可用的升级媒介
        else:
            self.set_default_sourceslist()

            if(self.launch_mode == LaunchMode.AUTOMATIC):
                self.exit_app("没有可用的升级媒介，程序退出.")
            else:
                self.animation_msg_box.set_animation_args("没有可用的升级媒介，程序退出.", True)
                self.animation_msg_box.showAnimation(5000)

    # 检测apt update、自我升级、系统升级
    def check_status(self):
        # 修改sourceslist
        self.available_source = self.kus.available_source
        lines = self.kus.get_sources()
        self.iface.change_source_list_to_kylin_update(lines)

        # 需要apt-get update
        if(self.kus.check_need_apt_get_update() == True):
            # 执行apt-get update，完成后由slot_dbus_process_msg() 执行 check_update_type()
            self.iface.call_update_source_by_shell()
        else:
            self.kus.check_update_type()

    # 任务分支
    def switch_task(self, is_kylin_update, is_kum_update):
        # pyqt缺陷，密集执行函数时初始化trayicon很容易导致trayicon大小异常
        # 不得已将trayicon的初始化放此处，在大部分任务处理完之后
        self.tray_icon = SystemTray(self)

        if(is_kum_update == True):
            self.task_mode = TaskMode.UPDATEKUM
        else:
            if(is_kylin_update == True):
                self.task_mode = TaskMode.UPDATEOS
            else:
                self.task_mode = TaskMode.UPDATEPKG

        # 更新管理器自我升级流程
        if(self.task_mode == TaskMode.UPDATEKUM):
            self.tray_icon.start_tray_msg("正在升级更新管理器.")
            self.is_working_c = True
            self.iface.call_upone("kylin-update-manager")

        # 系统升级流程
        elif(self.task_mode == TaskMode.UPDATEOS):
            # 提示进行系统升级
            self.show_to_frontend()
            (os_version_local, os_version_source) = self.kus.get_os_versions()
            qb = QMessageBox(self)
            qb.setIcon(QMessageBox.Question)
            qb.setWindowTitle("发现SP补丁包")
            qb.setText("当前SP版本 :  " + os_version_local + "\n可更新为 :  " + os_version_source + "\n\n是否现在执行更新？   ")
            qb.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            qb.setButtonText(QMessageBox.Yes, QString("升 级"))
            qb.setButtonText(QMessageBox.No, QString("取 消"))
            rtn = qb.exec_()

            if(rtn == QMessageBox.Yes):
                # 下载安装kylin-update包，进入系统升级流程
                self.resize(LOADINGWIDTH, LOADINGHEIGHT)
                self.loading_widget.progressBar.hide()
                self.loading_widget.loadinglabel.show()
                self.move_to_center()
                self.show_to_frontend()
                self.iface.call_insone("kylin-update-" + self.kus.desktop_or_server)

            # 用户不升级系统，进入包升级流程
            else:
                self.kus.start_access()

        # 包升级流程
        else:
            self.kus.start_access()

    # 自我升级完成，退出dbus，重启更新管理器
    def kylin_update_manager_upgrade_finish(self):
        if self.iface:
            self.iface.exit_dbus()

        if self.launch_mode == LaunchMode.AUTOMATIC:
            os.system("kylin-update-manager-restart -auto &")
        else:
            os.system("kylin-update-manager-restart -m &")

        self.close()
        sys.exit("更新管理器重启.")

    # 安装kylin-update完成，执行并退出更新管理器
    def kylin_update_install_finish(self):
        os.system("/tmp/kylin-update/kylin-simpleupdate &")
        self.exit_app("开始系统升级，更新管理器自动退出.")

    # 将sources.list重置为可用 + 默认
    def set_default_sourceslist(self):
        os_codename = self.cu.get_os_codename()
        current_user = self.cu.get_current_user()
        self.iface.change_source_list_to_default(self.service_key, self.hash_pwd, current_user, os_codename, self.available_source)

    '''
    Dbus Apt后台信号统一处理函数
    信号传递顺序：
    daemon.kum_fetch_signal() / kum_apt_signal()
    dbus_service.slot_kum_fetch_signal() / slot_kum_apt_signal()
    self.slot_dbus_process_msg()
    '''
    def slot_dbus_process_msg(self, signal_type, app_name, action, percent, process_type, status):
        print app_name, " : ", signal_type, " : ", process_type, " : ", str(percent), " : ", status, " : ", action

        # 错误信号
        if process_type == "apt_error":
            self.set_default_sourceslist()
            self.is_working = False
            if app_name == "#update" or app_name == "#update-shell":
                self.animation_msg_box.set_animation_args("源更新失败，程序将在5秒后退出.", True)
                self.animation_msg_box.showAnimation(5000)
            else:
                # SP升级流程时出错,自动退出.
                if(self.task_mode == TaskMode.UPDATEOS):
                    self.animation_msg_box.set_animation_args("SP补丁更新错误，程序将在5秒后退出.", True)
                    self.animation_msg_box.showAnimation(5000)
                else:
                    self.animation_msg_box.set_animation_args("软件更新失败，程序将在5秒后退出.", True)
                    self.animation_msg_box.showAnimation(5000)

        # 调用本地命令dpkg --configure -a过程的信号
        if app_name == "#configure-shell":
            if process_type == "finish" and str(percent) == "200.0":
                # dpkg --configure -a完成，开始执行main work
                self.loading_widget.tiplabel.setText("正在从源中更新数据...")
                self.main_work()

        # 调用本地命令apt-get update过程的信号
        if app_name == "#update-shell":
            if process_type == "finish" and str(percent) == "200.0":
                # apt-get update完成，开始检测是否有系统版本升级
                self.kus.check_update_type()

        # 更新管理器自我升级的信号，kylin-update-manager包安装完成，重启自身和dbus后台
        if app_name == "kylin-update-manager":
            if process_type == "apt_finish" and str(percent) == "200.0":
                self.kylin_update_manager_upgrade_finish()

        # 系统升级流程的信号，kylin-update包安装完成，执行系统升级
        if app_name == "kylin-update-desktop" or app_name == "kylin-update-server":
            if process_type == "apt_finish" and str(percent) == "200.0":
                self.kylin_update_install_finish()

        # 包升级过程的信号
        else:
            # 下载过程
            if app_name == "#install_and_upgrade" and signal_type == "fetch":
                if process_type == "down_start":
                    self.detail_widget.te_detail.append("开始下载.")
                    self.detail_widget.pb_fetch.setValue(0)
                elif process_type == "down_stop":
                    self.detail_widget.te_detail.append("下载完成，准备安装.")
                    self.detail_widget.pb_fetch.setValue(100)
                else:
                    if "down_pulse" in process_type:
                        # apt会重复打印信息，过滤掉
                        if process_type != self.previous_process_type:
                            self.previous_process_type = process_type
                            self.detail_widget.te_detail.append(process_type.split(" ")[1])
                    if percent not in [0.0, "0.0", None]:
                        self.detail_widget.pb_fetch.setValue(int(percent))
                        self.tray_icon.process_now = str(int(percent) / 2)
            # 安装过程
            elif app_name == "#install_and_upgrade" and signal_type == "apt" and process_type == "apt_start":
                self.detail_widget.pb_apt.setValue(0)
                self.detail_widget.te_detail.append(status)
                self.detail_widget.te_detail.append("正在安装...")
            elif app_name == "#install_and_upgrade" and signal_type == "apt" and process_type == "apt_finish":
                if self.is_working_c == True:
                    LogManager.INFO("关键更新完成...")
                    self.is_working_c = False
                    self.detail_widget.pb_fetch.setValue(0)
                    self.detail_widget.pb_apt.setValue(0)
                    self.detail_widget.te_detail.clear()
                    self.tray_icon.hide()

                    if self.isHidden() == True:
                        self.set_default_sourceslist()
                        self.animation_msg_box.set_animation_args("更新完成，程序将在5秒后退出.", True)
                        self.animation_msg_box.showAnimation(5000)
                else:
                    LogManager.INFO("普通和重要更新完成...")
                    self.is_working = False
                    self.detail_widget.pb_apt.setValue(100)
                    self.detail_widget.te_detail.append("更新完成.")
                    self.set_default_sourceslist()
                    self.animation_msg_box.set_animation_args("更新完成，程序将在5秒后退出.", True)
                    self.animation_msg_box.showAnimation(5000)
            else:
                if signal_type == "apt" and process_type == "apt_pulse":
                    self.detail_widget.pb_apt.setValue(int(percent))
                    self.tray_icon.process_now = str(50 + (int(percent) / 2))

    # 所有更新信息获取完成
    def slot_access_completed(self, size_byte_o, size_byte_i, len_o, len_i, crucial_list):
        # 没有任何更新
        if(len_o == 0 and len_i == 0 and len(crucial_list) == 0):
            self.hide()
            self.set_default_sourceslist()
            if self.launch_mode == LaunchMode.MANUAL:
                self.loading_widget.tiplabel.setText("没有需要更新的软件")
                self.animation_msg_box.set_animation_args("没有需要更新的软件，程序将自动退出.", True)
                self.animation_msg_box.showAnimation(5000)
            else:
                self.exit_app("没有可用的更新.")
        else:
            self.app_num_ordinary = len_o
            self.app_num_important = len_i
            self.selected_num_ordinary = len_o
            self.selected_num_important = len_i
            self.size_ordinary = size_byte_o
            self.size_important = size_byte_i

            # 优先安装关键更新
            if (len(crucial_list) > 0):
                if (self.launch_mode == LaunchMode.AUTOMATIC):
                    self.tray_icon.start_tray_msg("正在执行关键更新.")
                self.is_working_c = True
                self.iface.call_install_and_upgrade(crucial_list)

            if len_o > 0 and len_i > 0:
                self.change_display_content(LWMode.BOTH)
            elif len_o == 0 and len_i > 0:
                self.change_display_content(LWMode.IMPORTANT)
            elif len_o > 0 and len_i == 0:
                self.change_display_content(LWMode.ORDINARY)
            elif len_o == 0 and len_i == 0:
                # hide表示只有关键更新
                #???????????????????????????????????????
                self.hide()

            print "普通更新 : ", len_o, ",重要更新 : ", len_i, ",关键更新 : ", len(crucial_list)

    # 每获得一个app信息
    def slot_get_one_app(self, msg):
        app_msg = AppMsg()
        app_msg.ZH_CN_NAME = msg["displayName"]
        app_msg.NAME = msg["name"]
        app_msg.ICON = msg["icon"]
        app_msg.SIZE = msg["downloadSizeHuman"]
        app_msg.BYTESIZE = msg["downloadSize"]
        app_msg.INSTALLEDVERSION = msg["installedVersion"]
        app_msg.CANDIDATEVERSION = msg["candidateVersion"]
        app_msg.DESCRIPTION = msg["description"]
        if msg['apptype'] == AppType.ORDINARY:
            self.add_ordinary_item(app_msg)
        elif msg['apptype'] == AppType.IMPORTANT:
            self.add_important_item(app_msg)

    # 将一个普通app添加到ui
    def add_ordinary_item(self, app_msg):
        item = QListWidgetItem()
        list_item = UpdateListItemWidget(app_msg, self)
        list_item.setStatusTip(str(app_msg.BYTESIZE))
        item.setWhatsThis(app_msg.DESCRIPTION)
        list_item.size_change.connect(self.recalculate_ordinary_apps_size)
        list_item.checkBox.clicked.connect(self.set_checkbox_ordinary)
        self.bottom_widget.lw_ordinary.addItem(item)
        self.bottom_widget.lw_ordinary.setItemWidget(item, list_item)
        self.item_map_ordinary[app_msg.NAME] = list_item

    # 将一个重要app添加到ui
    def add_important_item(self, app_msg):
        item = QListWidgetItem()
        list_item = UpdateListItemWidget(app_msg, self)
        list_item.setStatusTip(str(app_msg.BYTESIZE))
        item.setWhatsThis(app_msg.DESCRIPTION)
        list_item.size_change.connect(self.recalculate_important_apps_size)
        list_item.checkBox.clicked.connect(self.set_checkbox_important)
        self.bottom_widget.lw_important.addItem(item)
        self.bottom_widget.lw_important.setItemWidget(item, list_item)
        self.item_map_important[app_msg.NAME] = list_item

    # ui内容初始化
    def change_display_content(self, lw_mode):
        self.top_widget.label_all_num.setText(str(self.app_num_ordinary + self.app_num_important))
        self.top_widget.label_selected_num.setText(str(self.selected_num_ordinary + self.selected_num_important))
        size_all_human = self.cu.byte_human(self.size_ordinary + self.size_important)
        self.top_widget.label_selected_size.setText(size_all_human)

        if lw_mode == LWMode.BOTH:
            size_ordinary_human = self.cu.byte_human(self.size_ordinary)
            size_important_human = self.cu.byte_human(self.size_important)
            self.bottom_widget.label_ordinary.setText("普通更新详情 (数量：" + str(self.app_num_ordinary) + "，已选中：" + str(self.selected_num_ordinary) + "项，大小：" + size_ordinary_human + ")")
            self.bottom_widget.label_important.setText("重要更新详情 (数量：" + str(self.app_num_important) + "，已选中：" + str(self.selected_num_important) + "项，大小：" + size_important_human + ")")
            self.bottom_widget.lw_ordinary.setCurrentRow(0)
            self.bottom_widget.lw_important.setCurrentRow(0)
            first_item = self.bottom_widget.lw_important.item(0)
            self.bottom_widget.te_description.setText(first_item.whatsThis())
            self.bottom_widget.show_both_title()
        elif lw_mode == LWMode.IMPORTANT:
            size_important_human = self.cu.byte_human(self.size_important)
            self.bottom_widget.label_important.setText("重要更新详情 (数量：" + str(self.app_num_important) + "，已选中：" + str(self.selected_num_important) + "项，大小：" + size_important_human + ")")
            self.bottom_widget.lw_important.setCurrentRow(0)
            first_item = self.bottom_widget.lw_important.item(0)
            self.bottom_widget.te_description.setText(first_item.whatsThis())
            self.bottom_widget.show_important_title()
        else:
            size_ordinary_human = self.cu.byte_human(self.size_ordinary)
            self.bottom_widget.label_ordinary.setText("普通更新详情 (数量：" + str(self.app_num_ordinary) + "，已选中：" + str(self.selected_num_ordinary) + "项，大小：" + size_ordinary_human + ")")
            self.bottom_widget.lw_ordinary.setCurrentRow(0)
            first_item = self.bottom_widget.lw_ordinary.item(0)
            self.bottom_widget.te_description.setText(first_item.whatsThis())
            self.bottom_widget.show_ordinary_title()

        self.show_main_window()
        self.switch_to_list_window()

    # description切换
    def change_description_content(self, status):
        if status:
            firestItem = self.bottom_widget.lw_important.item(self.bottom_widget.lw_important.currentRow())
            self.bottom_widget.te_description.setText(firestItem.whatsThis())
        else:
            firestItem = self.bottom_widget.lw_ordinary.item(self.bottom_widget.lw_ordinary.currentRow())
            self.bottom_widget.te_description.setText(firestItem.whatsThis())

    # 加减计算所选的普通升级包信息
    def recalculate_ordinary_apps_size(self, size_str, flag):
        if flag:
            self.size_ordinary += long(size_str)
            self.selected_num_ordinary += 1
        else:
            self.size_ordinary -= long(size_str)
            self.selected_num_ordinary -= 1

        self.top_widget.label_selected_num.setText(str(self.selected_num_ordinary + self.selected_num_important))
        size_all_human = self.cu.byte_human(self.size_ordinary + self.size_important)
        self.top_widget.label_selected_size.setText(size_all_human)
        size_ordinary_human = self.cu.byte_human(self.size_ordinary)
        self.bottom_widget.label_ordinary.setText("普通更新详情 (数量：" + str(self.app_num_ordinary) + "，已选中：" + str(self.selected_num_ordinary) + "项，大小：" + size_ordinary_human + ")")

    # 加减计算所选的重要升级包信息
    def recalculate_important_apps_size(self, size_str, flag):
        if flag:
            self.size_important += long(size_str)
            self.selected_num_important += 1
        else:
            self.size_important -= long(size_str)
            self.selected_num_important -= 1

        self.top_widget.label_selected_num.setText(str(self.selected_num_ordinary + self.selected_num_important))
        size_all_human = self.cu.byte_human(self.size_ordinary + self.size_important)
        self.top_widget.label_selected_size.setText(size_all_human)
        size_important_human = self.cu.byte_human(self.size_important)
        self.bottom_widget.label_important.setText("重要更新详情 (数量：" + str(self.app_num_important) + "，已选中：" + str(self.selected_num_important) + "项，大小：" + size_important_human + ")")

    # 改变普通升级区域checkbox的状态
    def set_checkbox_ordinary(self):
        count = len(self.item_map_ordinary)
        m = 0
        for item in self.item_map_ordinary:
            curItem = self.item_map_ordinary[item]
            if curItem.get_checkbox_status() == True:
                m += 1
        if m == 0:
            self.bottom_widget.cb_ordinary.setCheckboxStatus(0)
        elif m == count:
            self.bottom_widget.cb_ordinary.setCheckboxStatus(2)
        else:
            self.bottom_widget.cb_ordinary.setCheckboxStatus(1)

    # 改变重要升级区域checkbox的状态
    def set_checkbox_important(self):
        count = len(self.item_map_important)
        m = 0
        for item in self.item_map_important:
            curItem = self.item_map_important[item]
            if curItem.get_checkbox_status() == True:
                m += 1
        if m == 0:
            self.bottom_widget.cb_important.setCheckboxStatus(0)
        elif m == count:
            self.bottom_widget.cb_important.setCheckboxStatus(2)
        else:
            self.bottom_widget.cb_important.setCheckboxStatus(1)

    # 显示到前端
    def show_to_frontend(self):
        self.show()
        self.raise_()

    # 重复运行程序
    def single_work(self):
        if self.isHidden() == True:
            print "已经有一个程序在运行了."
            self.animation_msg_box.set_animation_args("已经有一个程序在运行了.", False)
            self.animation_msg_box.showAnimation(5000)
        else:
            self.show_to_frontend()

    # 退出程序
    def exit_app(self, msg=None):
        if self.iface:
            self.iface.exit_dbus()
        self.close()

        sys.exit(msg)

    def switch_to_list_window(self):
        self.resize(MAINWIDTH, MAINHEIGHT)
        self.move_to_center()
        self.mainGroup.start()

    def switch_to_detail_window(self):
        self.resize(DETAILWIDTH, DETAILHEIGHT)
        self.move_to_center()
        self.detailGroup.start()

    # 窗口居中
    def move_to_center(self):
        windowWidth = QApplication.desktop().width()
        windowHeight = QApplication.desktop().height()
        self.move((windowWidth - self.width()) / 2, (windowHeight - self.height()) / 2)

    def show_main_window(self):
        self.move_to_center()
        self.show_to_frontend()

    def slot_click_min_button(self):
        self.showMinimized()

    def slot_click_close_button(self):
        # 正在执行关键更新，点击关闭则hide主界面
        if self.is_working_c == True:
            self.hide()
        else:
            if self.is_working == True:
                self.animation_msg_box.set_animation_args("更新未完成，请稍等.", False)
                self.animation_msg_box.showAnimation(5000)
            else:
                self.set_default_sourceslist()
                self.exit_app()

    def slot_click_install_button(self):
        work_list = []
        for item in self.item_map_ordinary:
            one_item = self.item_map_ordinary[item]
            if one_item.get_checkbox_status():
                work_list.append(one_item.app_msg.NAME)
        for item in self.item_map_important:
            one_item = self.item_map_important[item]
            if one_item.get_checkbox_status():
                work_list.append(one_item.app_msg.NAME)

        if work_list:
            self.is_working = True
            self.switch_to_detail_window()
            self.iface.call_install_and_upgrade(work_list)
        else:
            self.animation_msg_box.set_animation_args("至少选择一个更新.", False)
            self.animation_msg_box.showAnimation(5000)

    def slot_click_update_item(self, item):
        description = item.whatsThis()
        self.bottom_widget.te_description.setText(description)

    def slot_reset_ordinary_cb(self, status):
        if status == 0:
            for item in self.item_map_ordinary:
                curItem = self.item_map_ordinary[item]
                curItem.checkBox.setCheckState(Qt.Unchecked)
        elif status == 2:
            for item in self.item_map_ordinary:
                curItem = self.item_map_ordinary[item]
                curItem.checkBox.setCheckState(Qt.Checked)

    def slot_reset_important_cb(self, status):
        if status == 0:
            for item in self.item_map_important:
                curItem = self.item_map_important[item]
                curItem.checkBox.setCheckState(Qt.Unchecked)
        elif status == 2:
            for item in self.item_map_important:
                curItem = self.item_map_important[item]
                curItem.checkBox.setCheckState(Qt.Checked)

    # 实现无标题栏窗口拖动
    def mousePressEvent(self, event):
        if(event.button() == Qt.LeftButton):
            self.mouse_press = True
            self.drag_pos = event.globalPos() - self.pos()
            event.accept()
    def mouseReleaseEvent(self, event):
        self.mouse_press = False
        self.setWindowOpacity(1)
    def mouseMoveEvent(self, event):
        if(self.mouse_press):
            self.move_pos = event.globalPos()
            self.move(self.move_pos - self.drag_pos)
            self.setWindowOpacity(0.9)
            event.accept()


def main():
    app = QApplication(sys.argv)
    QTextCodec.setCodecForTr(QTextCodec.codecForName("UTF-8"))
    QTextCodec.setCodecForCStrings(QTextCodec.codecForName("UTF-8"))

    globalfont = QFont()
    globalfont.setPixelSize(14)
    globalfont.setFamily("方正黑体_GBK")
    app.setFont(globalfont)

    app.setOrganizationName("Kylin")
    app.setOrganizationDomain("kylinos.cn")
    app.setApplicationName("kylin-update-manager")

    # 获取启动参数
    if(len(sys.argv) == 1):
        MainWindow(LaunchMode.MANUAL)
        sys.exit(app.exec_())
    elif(len(sys.argv) == 2):
        if(sys.argv[1] == "-m"):
            MainWindow(LaunchMode.MANUAL)
            sys.exit(app.exec_())
        elif(sys.argv[1] == "-auto"):
            MainWindow(LaunchMode.AUTOMATIC)
            sys.exit(app.exec_())
        else:
            print "参数错误...", sys.argv
    else:
        print "参数错误...", sys.argv

if __name__ == '__main__':
    main()
