#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = 'lixiang'


from PyQt4.QtGui import *
from PyQt4.QtCore import *
from utils.enums import (BUTTON_ENTER, BUTTON_LEAVE, BUTTON_PRESSED, BUTTON_DISABLE)

class SystemButton(QPushButton):

    def __init__(self, parent = None):
        super(SystemButton,self).__init__(parent)
        self.status = BUTTON_LEAVE

    def loadPixmap(self, pic_name):
        self.pixmap = QPixmap(pic_name)
        self.btn_width = self.pixmap.width()/3
        self.btn_height = self.pixmap.height()
        self.setFixedSize(self.btn_width, self.btn_height)

    def enterEvent(self,event):
        self.status = BUTTON_ENTER
        self.update()

    def mousePressEvent(self,event):
        if(event.button() == Qt.LeftButton):
            self.mouse_press = True
            self.status = BUTTON_PRESSED
            self.update()

    def mouseReleaseEvent(self,event):
        if(self.mouse_press and self.rect().contains(event.pos())):
            self.mouse_press = False
            self.status = BUTTON_ENTER
            self.update()
            self.clicked.emit(True)

    def leaveEvent(self,event):
        self.status = BUTTON_LEAVE
        self.update()

    def paintEvent(self,event):
        self.painter = QPainter()
        self.painter.begin(self)
        self.painter.drawPixmap(self.rect(), self.pixmap.copy(self.btn_width * self.status, 0, self.btn_width, self.btn_height))
        self.painter.end()


class CleanButton(QWidget):

    clicked = pyqtSignal()

    checkboxList = []
    buttonStatus = None
    tristateStatus = None

    def __init__(self, parent=None):
        super(CleanButton,self).__init__(parent)
        # self.status = BUTTON_LEAVE
        self.init_ui()

    def init_ui(self):
        self.setFixedSize(12, 12)
        pixMap = QPixmap("./res/checkbox-tristate.png")
        width = pixMap.width()
        height = pixMap.height()
        num = 9
        for i in range(0, num):
            self.checkboxList.append(pixMap.copy(i*(width/num), 0, width/num, height))

    def setCleanButtonStatus(self, status):
        self.buttonStatus = status
        self.update()

    def setCheckboxStatus(self ,status):
        self.tristateStatus = status
        self.setCleanButtonStatus(BUTTON_LEAVE)

    def paintEvent(self, event):
        painter = QPainter(self)
        if self.buttonStatus == BUTTON_LEAVE:
            if self.tristateStatus == 2:
                painter.drawPixmap(self.rect(), self.checkboxList[3])
            elif self.tristateStatus == 1:
                painter.drawPixmap(self.rect(), self.checkboxList[6])
            elif self.tristateStatus == 0:
                painter.drawPixmap(self.rect(), self.checkboxList[0])
        elif self.buttonStatus == BUTTON_ENTER:
            if self.tristateStatus == 2:
                painter.drawPixmap(self.rect(), self.checkboxList[4])
            elif self.tristateStatus == 1:
                painter.drawPixmap(self.rect(), self.checkboxList[7])
            elif self.tristateStatus == 0:
                painter.drawPixmap(self.rect(), self.checkboxList[1])
        elif self.buttonStatus == BUTTON_PRESSED:
            if self.tristateStatus == 2:
                painter.drawPixmap(self.rect(), self.checkboxList[4])
            elif self.tristateStatus == 1:
                painter.drawPixmap(self.rect(), self.checkboxList[7])
            elif self.tristateStatus == 0:
                painter.drawPixmap(self.rect(), self.checkboxList[1])

    def enterEvent(self, event):
        self.setCleanButtonStatus(BUTTON_ENTER)

    def leaveEvent(self, event):
        self.setCleanButtonStatus(BUTTON_LEAVE)

    def mousePressEvent(self ,event):
        if event.button() == Qt.LeftButton:
            self.setCleanButtonStatus(BUTTON_PRESSED)
        else:
            event.ignore()

    def mouseReleaseEvent(self ,event):
        if event.button() == Qt.LeftButton:
            if self.rect().contains(event.pos()):
                self.setCleanButtonStatus(BUTTON_ENTER)
                self.clicked.emit()
            else:
                self.setCleanButtonStatus(BUTTON_LEAVE)
        else:
            event.ignore()

if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
    btn = SystemButton()
    btn.loadPixmap("../res/close_button.png")
    btn.show()
    sys.exit(app.exec_())