#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = 'lixiang'

from PyQt4.QtGui import *
from PyQt4.QtCore import *
from systembutton import SystemButton
from utils.enums import VERSION


class TopWidget(QWidget):

    m_parent = None

    def __init__(self, parent=None):
        super(TopWidget, self).__init__(parent)

        self.m_parent = parent
        self.setGeometry(0, 0, 500, 141)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.init_ui()
        self.init_style_sheet()
        self.init_text()

    def init_ui(self):
        self.setAutoFillBackground(True)
        palette = QPalette()
        palette.setColor(QPalette.Background, QColor(15,132,188))
        self.setPalette(palette)

        self.btn_min = SystemButton(self)
        self.btn_min.loadPixmap("./res/min_button.png")
        self.btn_min.setGeometry(428, 0, 36, 36)

        self.btn_close = SystemButton(self)
        self.btn_close.loadPixmap("./res/close_button.png")
        self.btn_close.setGeometry(464, 0, 36, 36)

        self.label_title = QLabel(self)
        self.label_title.setGeometry(50, 10, 260, 16)

        self.label_1 = QLabel(self)
        self.label_all_num = QLabel(self)
        self.label_2 = QLabel(self)
        self.label_all_num.setAlignment(Qt.AlignCenter)
        self.label_all_num.adjustSize()

        self.label_3 = QLabel(self)
        self.label_selected_num = QLabel(self)
        self.label_4 = QLabel(self)
        self.label_selected_size = QLabel(self)
        self.label_selected_num.setAlignment(Qt.AlignCenter)
        self.label_selected_num.adjustSize()
        self.label_selected_size.adjustSize()

        self.label_logo = QLabel(self)
        self.label_logo.setGeometry(8, 8, 42, 41)
        self.label_logo.setFrameShape(QFrame.NoFrame)
        self.label_logo.setAutoFillBackground(True)
        self.label_logo.setScaledContents(True)
        pixmap_logo = QPixmap("./res/logo.png")
        self.label_logo.setPixmap(pixmap_logo)

        self.label_img = QLabel(self)
        self.label_img.setFixedSize(72, 72)
        self.label_img.setFrameShape(QFrame.NoFrame)
        self.label_img.setAutoFillBackground(True)
        self.label_img.setScaledContents(True)
        pixmap_img = QPixmap("./res/update.png")
        self.label_img.setPixmap(pixmap_img)

        self.btn_install = QPushButton(self)
        self.btn_install.setCursor(Qt.PointingHandCursor)
        self.btn_install.setFocusPolicy(Qt.NoFocus)
        self.btn_install.setGeometry(331, 55, 150, 50)

        self.btn_delay = QPushButton(self)
        self.btn_delay.setCursor(Qt.PointingHandCursor)
        self.btn_delay.setFocusPolicy(Qt.NoFocus)
        self.btn_delay.setGeometry(390, 105, 91, 27)
        # 暂不启用
        self.btn_delay.hide()

        layout_h_all = QHBoxLayout()
        layout_h_all.addWidget(self.label_1)
        layout_h_all.addWidget(self.label_all_num)
        layout_h_all.addWidget(self.label_2)
        layout_h_all.addStretch()
        layout_h_all.setSpacing(1)
        layout_h_all.setMargin(0)
        layout_h_all.setContentsMargins(0, 0, 0, 0)

        layout_h_select = QHBoxLayout()
        layout_h_select.addWidget(self.label_3)
        layout_h_select.addWidget(self.label_selected_num)
        layout_h_select.addWidget(self.label_4)
        layout_h_select.addWidget(self.label_selected_size)
        layout_h_select.addStretch()
        layout_h_select.setSpacing(1)
        layout_h_select.setMargin(0)
        layout_h_select.setContentsMargins(0, 0, 0, 0)

        layout_v_label = QVBoxLayout()
        layout_v_label.addLayout(layout_h_all)
        layout_v_label.addLayout(layout_h_select)
        layout_v_label.addStretch()
        layout_v_label.setSpacing(10)
        layout_v_label.setMargin(0)
        layout_v_label.setContentsMargins(0, 20, 0, 0)

        layout_main = QHBoxLayout()
        layout_main.addWidget(self.label_img, 0, Qt.AlignVCenter)
        layout_main.addLayout(layout_v_label)
        layout_main.setSpacing(10)
        layout_main.setContentsMargins(10, 30, 0, 0)
        self.setLayout(layout_main)

    def init_style_sheet(self):
        self.label_title.setStyleSheet("QLabel{color:#ffffff;font-size:14px;}")
        self.label_1.setStyleSheet("QLabel{color:#ffffff;font-size:14px;}")
        self.label_all_num.setStyleSheet("QLabel{color:#f97150;font-size:30px;}")
        self.label_2.setStyleSheet("QLabel{color:#ffffff;font-size:14px;}")
        self.label_3.setStyleSheet("QLabel{color:#ffffff;font-size:12px;}")
        self.label_selected_num.setStyleSheet("QLabel{color:#f97150;font-size:12px;}")
        self.label_4.setStyleSheet("QLabel{color:#ffffff;font-size:12px;}")
        self.label_selected_size.setStyleSheet("QLabel{color:#ffffff;font-size:12px;}")
        self.btn_install.setStyleSheet("QPushButton{border:0px;font-size:22px;background:#52c832;color:#ffffff;}QPushButton:hover{background-color:#68d949;color:#ffffff;}QPushButton:pressed{background-color:#40b222;color:#ffffff;}")
        self.btn_delay.setStyleSheet("QPushButton{background:transparent;font-size:12px;color:#ffffff;text-align:right;}QPushButton:hover{color:#4fc3f7;}QPushButton:pressed{color:#4fc3f7;}")

    def init_text(self):
        self.label_title.setText("麒麟更新管理器 " + VERSION)
        self.label_1.setText("共有 ")
        self.label_all_num.setText("  ")
        self.label_2.setText(" 项可选更新")
        self.label_3.setText("已选中 ")
        self.label_selected_num.setText("  ")
        self.label_4.setText(" 项，共占磁盘 ")
        self.label_selected_size.setText("  ")
        self.btn_install.setText("确认更新")
        font = self.btn_delay.font()
        font.setUnderline(True)
        self.btn_delay.setFont(font)
        self.btn_delay.setText("稍后提醒")
